@@ -69,7 +69,7 @@ REQ_ID_HEADER = 'X-OpenStack-Request-ID'
 # tell keystoneclient that we can ignore the /v1|v2/{project_id} component of
 # the service catalog when doing discovery lookups
 for svc in ('volume', 'volumev2', 'volumev3'):
-    discover.add_catalog_discover_hack(svc, re.compile('/v[12]/\w+/?$'), '/')
+    discover.add_catalog_discover_hack(svc, re.compile(r'/v[12]/\w+/?$'), '/')
 
 
 def get_server_version(url):
@@ -154,15 +154,16 @@ def _log_request_id(logger, resp, service_name):
 class SessionClient(adapter.LegacyJsonAdapter):
 
     def __init__(self, *args, **kwargs):
-        self.api_version = kwargs.pop('api_version', None)
-        self.api_version = self.api_version or api_versions.APIVersion()
+        apiver = kwargs.pop('api_version', None) or api_versions.APIVersion()
+        if not isinstance(apiver, api_versions.APIVersion):
+            apiver = api_versions.APIVersion(str(apiver))
+        if apiver.ver_minor != 0:
+            kwargs['default_microversion'] = apiver.get_string()
         self.retries = kwargs.pop('retries', 0)
         self._logger = logging.getLogger(__name__)
         super(SessionClient, self).__init__(*args, **kwargs)
 
     def request(self, *args, **kwargs):
-        kwargs.setdefault('headers', kwargs.get('headers', {}))
-        api_versions.update_headers(kwargs["headers"], self.api_version)
         kwargs.setdefault('authenticated', False)
 
         # Note(tpatil): The standard call raises errors from
@@ -209,7 +210,13 @@ class SessionClient(adapter.LegacyJsonAdapter):
 
     def _get_base_url(self):
         endpoint = self.get_endpoint()
-        base_url = '/'.join(endpoint.split('/')[:3]) + '/'
+        m = re.search('(.+)/v[1-3].*', endpoint)
+        if m:
+            # Get everything up until the version identifier
+            base_url = '%s/' % m.group(1)
+        else:
+            # Fall back to the root of the URL
+            base_url = '/'.join(endpoint.split('/')[:3]) + '/'
         return base_url
 
     def get_volume_api_version_from_endpoint(self):
@@ -279,7 +286,7 @@ class HTTPClient(object):
                 raise exceptions.EndpointNotFound()
 
         self.auth_url = auth_url.rstrip('/') if auth_url else None
-        self.version = 'v1'
+        self.ks_version = 'v1'
         self.region_name = region_name
         self.endpoint_type = endpoint_type
         self.service_type = service_type
@@ -479,6 +486,7 @@ class HTTPClient(object):
 
     def _extract_service_catalog(self, url, resp, body, extract_token=True):
         """See what the auth service told us and process the response.
+
         We may get redirected to another site, fail or actually get
         back a service catalog with a token and our endpoints.
         """
@@ -513,7 +521,7 @@ class HTTPClient(object):
                 raise
 
         elif resp.status_code == 305:
-            return resp['location']
+            return resp.headers['location']
         else:
             raise exceptions.from_response(resp, body)
 
@@ -550,7 +558,7 @@ class HTTPClient(object):
         path_parts = path.split('/')
         for part in path_parts:
             if len(part) > 0 and part[0] == 'v':
-                self.version = part
+                self.ks_version = part
                 break
 
         # TODO(sandy): Assume admin endpoint is 35357 for now.
@@ -560,7 +568,7 @@ class HTTPClient(object):
                                          path, query, frag))
 
         auth_url = self.auth_url
-        if self.version == "v2.0" or self.version == "v3":
+        if 'v2' in self.ks_version or 'v3' in self.ks_version:
             while auth_url:
                 if not self.auth_system or self.auth_system == 'keystone':
                     auth_url = self._v2_or_v3_auth(auth_url)
@@ -619,7 +627,7 @@ class HTTPClient(object):
 
     def _v2_or_v3_auth(self, url):
         """Authenticate against a v2.0 auth service."""
-        if self.version == "v3":
+        if self.ks_version == "v3":
             body = {
                 "auth": {
                     "identity": {
@@ -646,11 +654,11 @@ class HTTPClient(object):
                 body['auth']['tenantName'] = self.projectid
             elif self.tenant_id:
                 body['auth']['tenantId'] = self.tenant_id
-        self._authenticate(url, body)
+        return self._authenticate(url, body)
 
     def _authenticate(self, url, body):
         """Authenticate and extract the service catalog."""
-        if self.version == 'v3':
+        if self.ks_version == 'v3':
             token_url = url + "/auth/tokens"
         else:
             token_url = url + "/tokens"
