@@ -19,7 +19,6 @@ from __future__ import print_function
 import argparse
 import collections
 import os
-import sys
 
 from oslo_utils import strutils
 import six
@@ -31,7 +30,7 @@ from cinderclient import exceptions
 from cinderclient import shell_utils
 from cinderclient import utils
 
-from cinderclient.v2.shell import *  # flake8: noqa
+from cinderclient.v2.shell import *  # noqa
 
 FILTER_DEPRECATED = ("This option is deprecated and will be removed in "
                      "newer release. Please use '--filters' option which "
@@ -681,8 +680,9 @@ def do_create(cs, args):
 
     if args.poll:
         timeout_period = os.environ.get("POLL_TIMEOUT_PERIOD", 3600)
-        shell_utils._poll_for_status(cs.volumes.get, volume.id, info, 'creating', ['available'],
-                timeout_period, cs.client.global_request_id, cs.messages)
+        shell_utils._poll_for_status(
+            cs.volumes.get, volume.id, info, 'creating', ['available'],
+            timeout_period, cs.client.global_request_id, cs.messages)
 
     utils.print_dict(info)
 
@@ -1352,6 +1352,20 @@ def do_group_list(cs, args):
     columns = ['ID', 'Status', 'Name']
     utils.print_list(groups, columns)
 
+    with cs.groups.completion_cache(
+            'uuid',
+            cinderclient.v3.groups.Group,
+            mode='w'):
+        for group in groups:
+            cs.groups.write_to_completion_cache('uuid', group.id)
+    with cs.groups.completion_cache('name',
+                                    cinderclient.v3.groups.Group,
+                                    mode='w'):
+        for group in groups:
+            if group.name is None:
+                continue
+            cs.groups.write_to_completion_cache('name', group.name)
+
 
 @api_versions.wraps('3.13')
 @utils.arg('--list-volume',
@@ -1415,6 +1429,17 @@ def do_group_create(cs, args):
     info.pop('links', None)
     utils.print_dict(info)
 
+    with cs.groups.completion_cache('uuid',
+                                    cinderclient.v3.groups.Group,
+                                    mode='a'):
+        cs.groups.write_to_completion_cache('uuid', group.id)
+
+    if group.name is not None:
+        with cs.groups.completion_cache('name',
+                                        cinderclient.v3.groups.Group,
+                                        mode='a'):
+            cs.groups.write_to_completion_cache('name', group.name)
+
 
 @api_versions.wraps('3.14')
 @utils.arg('--group-snapshot',
@@ -2155,6 +2180,8 @@ def do_attachment_list(cs, args):
                                       marker=args.marker,
                                       limit=args.limit,
                                       sort=args.sort)
+    for attachment in attachments:
+        setattr(attachment, 'server_id', getattr(attachment, 'instance', None))
     columns = ['ID', 'Volume ID', 'Status', 'Server ID']
     if args.sort:
         sortby_index = None
